package implementation;

import java.io.File;
import java.io.FileNotFoundException;
import java.security.InvalidParameterException;
import java.util.*;

/**
 * Liest eine Datei in dem in der Aufgabenstellung spezifizierten Format ein
 */
public class FileInput {
	/**
	 * Liest die gegebene Datei als Spezifikation eines Holzpuzzles ein
	 *
	 * @param inputFile Inputdatei
	 * @return Eingelesenes Holzpuzzle
	 */
	// TODO: Fehlercheck notwendig?
	public HolzPuzzle read(File inputFile) {
		//Inititalisieren
		List<String> kommentare = new ArrayList<>();
		Wuerfel[][][] quader;
		List<Wuerfel> wuerfel = new ArrayList<>();

		//Einlesen
		try (Scanner s = new Scanner(inputFile)) {
			//Kommentare
			String zeile = s.nextLine();
			while (zeile.startsWith("//") && s.hasNextLine()) {
				kommentare.add(zeile);
				zeile = s.nextLine();
			}

			//Dimensionen
			String[] dimensionen = zeile.toLowerCase().replace("dimension", "").trim().split(",");
			if (dimensionen.length != 3) {
				throw new InvalidParameterException("Falsche Anzahl an Dimensionen, nur exakt drei Erlaubt");
			}
			int[] dim = new int[3];
			for (int i = 0; i < dim.length; i++) {
				try {
					dim[i] = Integer.parseInt(dimensionen[i]);
					if (dim[i] < 1) {
						throw new InvalidParameterException("Dimension " + (i + 1) + " ist nicht valide. Die Dimension muss >0 sein");
					}
				} catch (NumberFormatException e) {
					throw new InvalidParameterException("Dimension " + (i + 1) + " ist keine Zahl");
				}
			}
			int produktDim = dim[0] * dim[1] * dim[2];
			quader = new Wuerfel[dim[0]][dim[1]][dim[2]];
			Set<String> allNames = new HashSet<>();
			//Wuerfel
			while (s.hasNextLine()) {
				zeile = s.nextLine();
				String name = zeile.substring(0, zeile.lastIndexOf(":"));
				String[] werte = zeile.substring(zeile.lastIndexOf(":") + 1).trim().split(" ");
				if (werte.length != 6) {
					throw new InvalidParameterException("Mindestens ein Wuerfel hat nicht genau 6 Seiten");
				}
				int[] wert = new int[6];
				for (int i = 0; i < werte.length; i++) {
					wert[i] = Integer.parseInt(werte[i]);
					if (wert[i] < 0 | wert[i] > 4) {
						throw new InvalidParameterException("Falscher Seitenwert fuer eine Wuerfelseite");
					}
				}
				wuerfel.add(new Wuerfel(name, wert));
				if (allNames.contains(name)) {
					throw new InvalidParameterException("Alle Wuerfelnamen muessen individuell sein");
				}
				allNames.add(name);
			}
			if (wuerfel.size() != produktDim) {
				throw new InvalidParameterException("Falsche Anzahl Wuerfel, Wuerfelanzahl muss " + produktDim + " betragen.");
			}
		} catch (NoSuchElementException | FileNotFoundException f) {
			throw new InvalidParameterException("Eingabedatei nicht lesbar");
		}

		//Ausgabe
		return new HolzPuzzle(wuerfel, quader, kommentare);
	}
}
