package implementation;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Paths;
import java.util.Scanner;

/**
 * Gibt das Holzpuzzle in eine Datei aus.
 */
public class FileOutput {
	/**
	 * Druckt das geloeste Puzzle in eine Datei aus. Wenn auf die Datei nicht zugegriffen werden kann
	 * wird ein neuer Pfad abgefragt (um die Daten nicht zu verlieren). Wurde keine Loesung gefunden,
	 * werden die Teile mit dem Zusatz "Keine Loesung gefunden" ausgegeben
	 *
	 * @param holzPuzzle Puzzle geloest oder ungeloest
	 * @param outputFile Ausgabedatei
	 */
	public void print(HolzPuzzle holzPuzzle, File outputFile) throws IOException {
		outputFile.getAbsoluteFile().getParentFile().mkdirs();
		FileWriter output = null;
		try {
			output = new FileWriter(outputFile);
			for (String s : holzPuzzle.getKommentare()) {
				output.write(s + System.lineSeparator());
			}
			output.write("Dimension ");
			Wuerfel[][][] quader = holzPuzzle.getQuader();
			output.write(quader.length + ",");
			output.write(quader[0].length + ",");
			output.write(quader[0][0].length + System.lineSeparator());

			if (quader[0][0][0] != null) { //Ist ein Element im Quader leer, ist alles leer
				for (int i = 0; i < quader.length; i++) {
					for (int j = 0; j < quader[0].length; j++) {
						for (int k = 0; k < quader[0][0].length; k++) {
							output.write("[" + (i + 1) + "," + (j + 1) + "," + (k + 1) + "] " + quader[i][j][k].toString() + System.lineSeparator());
						}
					}
				}
			} else {
				for (Wuerfel wuerfel : holzPuzzle.getTeile()) {
					output.write(wuerfel.toString() + System.lineSeparator());
				}
			}
		} finally {
			if (output != null) {
				output.close();
			}
		}
	}
}
