package implementation;

import java.util.Arrays;
import java.util.Objects;

/**
 * Repraesentiert einen Wuerfel, der seitlich Dreiecke hat
 */
public class Wuerfel {
	//Attribute
	protected String name;
	private int[] werte;

	/**
	 * Erstellt einen neuen Wuerfel mit den mitgegebenen Parametern
	 * @param name Name des Wuerfels
	 * @param werte Positionen der Dreiecke auf den Seitenflaechen
	 */
	public Wuerfel(String name, int[] werte) {
		this.name = name;
		this.werte = werte;
	}

	/**
	 * Erstellt einen Klon des mitgegebenen Wuerfels
	 * @param w Vorbild fuer den Klon
	 */
	public Wuerfel(Wuerfel w) {
		this.name = w.name;
		this.werte = w.getWerte().clone();
	}

	/**
	 * Gibt die Stringrepraesentation des Wuerfels zurueck
	 * @return Stringrepresentation
	 */
	public String toString() {
		String wertString = "";
		boolean isFirst = true;
		for (int wert : werte) {
			if (isFirst) {
				isFirst = false;
			} else {
				wertString += " ";
			}
			wertString += wert;
		}
		return name + ": " + wertString;
	}

	/**
	 * Gibt die Seitenwerte zurueck
	 * @return Seitenwerte
	 */
	public int[] getWerte() {
		return werte;
	}

	public int getAussenflaechen() {
		int aussenCounter = 0;
		for (int wert : werte) {
			if (wert == 0) {
				aussenCounter++;
			}
		}
		return aussenCounter;
	}

	/**
	 * Dreht an der X-Achse um -90 Grad
	 */
	public void dreheX() {
		int temp = werte[0];
		werte[0] = werte[3] != 0 ? (2 + werte[3]) % 4 + 1 : 0;
		werte[3] = werte[5] != 0 ? (2 + werte[5]) % 4 + 1 : 0;
		werte[5] = werte[1] != 0 ? (2 + werte[1]) % 4 + 1 : 0;
		werte[1] = temp != 0 ? (2 + temp) % 4 + 1 : 0;
		werte[2] = werte[2] != 0 ? (2 + werte[2]) % 4 + 1 : 0;
		werte[4] = werte[4] != 0 ? werte[4] % 4 + 1 : 0;
	}

	/**
	 *Dreht an der Y-Achse um -90 Grad
	 */
	public void dreheY() {
		int temp = werte[0];
		werte[0] = werte[4] != 0 ? (werte[4] + 1) % 4 + 1 : 0;
		werte[4] = werte[5] != 0 ? (werte[5] + 1) % 4 + 1 : 0;
		werte[5] = werte[2];
		werte[2] = temp;
		werte[1] = werte[1] != 0 ? werte[1] % 4 + 1 : 0;
		werte[3] = werte[3] != 0 ? (2 + werte[3]) % 4 + 1 : 0;
	}

	/**
	 * Dreht an der Z-Achse um 90 Grad
	 */
	public void dreheZ() {
		int temp = werte[1];
		werte[1] = werte[4];
		werte[4] = werte[3];
		werte[3] = werte[2];
		werte[2] = temp;
		werte[0] = werte[0] != 0 ? (2 + werte[0]) % 4 + 1 : 0;
		werte[5] = werte[5] != 0 ? werte[5] % 4 + 1 : 0;
	}

	@Override
	public boolean equals(Object partyNextDoor){
		if (partyNextDoor instanceof Wuerfel){
			return partyNextDoor.hashCode() == this.hashCode();
		}
		return false;
	}

	@Override
	public int hashCode(){
		return Arrays.hashCode(werte);
	}

}